#include "includes.h"
#include "revinetd.h"
#include "relay_agt.h"
#include "misc.h"
#include "proxy.h"

static const char cvsid[] = "$Id: relay_agt.c,v 1.21 2003/04/09 22:10:52 alexcv Exp $";

int
relay_agent(char *callback, int port, char *target, int port2)
{
    int comm_chan, retval, tmp_sock, message = -1;
    time_t timer;
    struct sockaddr_in sa_comm, tmp_sa;
    struct timeval timeout;
    fd_set active, read;
    Channels *tmp_chan;

    /* init time. */
    timer = time(NULL);
    
    /* Establish the callback channel. */

    comm_chan = init_sockaddr(&sa_comm, callback, port);
    if (connect(comm_chan, (struct sockaddr *)&sa_comm,
                sizeof(sa_comm)) == -1) {
        perror("connect");
        return 1;
    }
    register_sock(comm_chan);

    if (conf.verbosity != VB_QUIET)
        printf("Server comm channel connected on %s:%i\n", callback, port);

    /* Initialize the channels */
    chan = NULL;

    FD_ZERO(&active);
    FD_SET(comm_chan, &active);

    while (1) {
        read = active;
        timeout.tv_sec = 180L; /* 3 minutes, as a long */
        timeout.tv_usec = 0L;  /* 0 micro seconds, also as a long */
        if ((retval = select(FD_SETSIZE, &read, NULL, NULL, &timeout)) < 0) {
            perror("select");
            exit(EXIT_FAILURE);
        }
        if (retval == 0) { /* timeout. */
            /* If the heart beat fail, the socket should throw us an
             * error. */
            send_comm_message(comm_chan, SV_HEART_BEAT);
            timer = time(NULL);
            continue;
        }
        if ((timer + 180) < time(NULL)) {
            send_comm_message(comm_chan, SV_HEART_BEAT);
            timer = time(NULL);
        }
        if (FD_ISSET(comm_chan, &read)) {
            if (conf.verbosity == VB_VERBOSE)
                printf("Received message on comm channel: ");

            message = get_comm_message(comm_chan);
            /* We either get a heart beat reply or a request for new
             * channel. */

            if (message == RA_TARGET_UP) {
                tmp_sock = init_sockaddr(&tmp_sa, callback, port);
                if (connect(tmp_sock, (struct sockaddr *)&tmp_sa,
                            sizeof(tmp_sa)) == -1) {
                    perror("connect");
                    return 1;
                }
                register_sock(tmp_sock);

                if (conf.verbosity != VB_QUIET)
                    printf("New backhaul connection %s:%i\n", callback, port);
                
                if (chan == NULL) {
                    chan = chan_add();
                    tmp_chan = chan;
                } else {
                    tmp_chan = chan;
                    while (tmp_chan->next != NULL) {
                        tmp_chan = tmp_chan->next;
                    }
                    tmp_chan->next = chan_add();
                    tmp_chan->next->prev = tmp_chan;
                    tmp_chan = tmp_chan->next;
                }

                tmp_chan->source = tmp_sock;

                tmp_sock = init_sockaddr(&tmp_sa, target, port2);
                if (connect(tmp_sock, (struct sockaddr *)&tmp_sa,
                            sizeof(tmp_sa)) == -1) {
                    perror("connect");
                    return 1;
                }
                register_sock(tmp_sock);

                if (conf.verbosity != VB_QUIET)
                    printf("New target connection %s:%i\n", target, port2);
            
                tmp_chan->target = tmp_sock;

                FD_SET(tmp_chan->source, &active);
                FD_SET(tmp_chan->target, &active);
            }
        }
        proxy(&read, &active);
    }
    
    return 0;
}
